/* $Id: et6000_text16.c,v 1.4 1998/11/24 18:28:10 marcus Exp $
******************************************************************************

   Tseng Labs ET6000 text16 operators

   Copyright (C) 1997 Jason McMullan	[jmcc@ggi-project.org]
   Copyright (C) 1997 Jim Ursetto	[jim.ursetto@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************

	This code was derived from the following source of information:
	[1]   IBM VGA driver by Jason McMullan

******************************************************************************
*/

#undef  DEBUG_P		/* Debug Font-pointer (_very_ verbose!) */

#include <IBM/vga.h>
#include <Tseng/et6000_io.h>
#include <Tseng/et6000_text16.h>
#include <kgi/system.h>

#include "pointer_mask.h"

void text16_data_init(void)
{
	memset(text16_data, 0, sizeof(struct text16_data *) * MAX_NR_DISPLAYS);
}

struct text16_data_s *text16_data_realloc(struct text16_data_s *old)
{
	int i;

	if (old == NULL || old->magic != TEXT16_MAGIC) {
		for (i = 0; i < MAX_NR_DISPLAYS; i++) 
			if (text16_data[i].magic != TEXT16_MAGIC) {
				memset(&(text16_data[i]), 0,
				 sizeof(struct text16_data_s));
				text16_data[i].magic = TEXT16_MAGIC;
				return &(text16_data[i]);
			}
		return NULL;
	}
	memset(old, 0, sizeof(struct text16_data_s));
	old->magic = TEXT16_MAGIC;
	return old;
}

/* Text-font based mouse pointer code */

/* Ascii codes of font locs. to abuse for our purposes */
static kgiu8 pointer_char[PCELLTOTAL] = {
	0xC5, 0xC6, 0xC7,
	0xD6, 0xD7, 0xD8,
	0xCF, 0xDD, 0xDE
};

#if DEBUG_LEVEL > 0
static void print_mask(const char *msg, uint *mask, uint texty)
{
	int i, j;
	char buff[33];

	printk("%s\n", msg);

	buff[32] = 0;
	for (i = 0; i < texty * 2; i++) {
		for (j = 0; j < 32; j++) 
			buff[31 - j] = (mask[i] & (1 << j)) ? 'O' : ' ';    
		printk("  [%.2d]%s\n", i, buff);
	}
}
#endif /* DEBUG_LEVEL > 0 */

typedef struct text16_font_reg_s 
{
	kgiu8 SR2,SR4;
	kgiu8 GR4,GR5,GR6;
} text16_font_reg;

#define FONT_GET 0
#define FONT_SET 1

#ifdef NO_LONGER_USED_CODE
static void text16_font_access(int getset, int chr, kgiu8 *charcell)
{
	text16_font_reg regs;
	kgiu8 *fontbuff = (kgiu8 *)(phys_to_virt(0xA0000)) + (chr << 5);
	IS_CRITICAL;

#ifdef SLOW_POINTER
	WAIT_VRETRACE();
#endif
	ENTER_CRITICAL;
	regs.SR2 = SEQ_in8(0x02);
	regs.SR4 = SEQ_in8(0x04);	/* Save register state        */
	regs.GR4 = GRC_in8(0x04);
	regs.GR5 = GRC_in8(0x05);
	regs.GR6 = GRC_in8(0x06);

    SEQ_out8(SR02_PLANE2, 0x02);		/* Font data goes to plane 2  */
    SEQ_out8(SR04_256K_ACCESS | SR04_NO_ODDEVEN, 0x04);

    GRC_out8(2, 0x04);			/* Read plane 2               */
    GRC_out8(GR5_WRITEMODE0, 0x05);
    GRC_out8(GR6_MAP_A0_64 | GR6_GRAPHMODE, 0x06);

    if (getset == FONT_GET) 
		memcpy(charcell, fontbuff, 32);
    else
		memcpy(fontbuff, charcell, 32);

	SEQ_out8(regs.SR2, 0x02);
	SEQ_out8(regs.SR4, 0x04);	/* Restore registers  */
	GRC_out8(regs.GR4, 0x04);
	GRC_out8(regs.GR5, 0x05);
	GRC_out8(regs.GR6, 0x06);
	LEAVE_CRITICAL;
}
#endif /* NO_LONGER_USED_CODE */

static inline void text16_font_enter(text16_font_reg *regs) 
{
	IS_CRITICAL;

#ifdef SLOW_POINTER
	WAIT_VRETRACE();
#endif
	ENTER_CRITICAL;
	regs->SR2 = SEQ_in8(0x02);
	regs->SR4 = SEQ_in8(0x04);	/* Save register state        */
	regs->GR4 = GRC_in8(0x04);
	regs->GR5 = GRC_in8(0x05);
	regs->GR6 = GRC_in8(0x06);

	SEQ_out8(SR02_PLANE2, 0x02);	/* Font data goes to plane 2  */
	SEQ_out8(SR04_256K_ACCESS | SR04_NO_ODDEVEN, 0x04);

	GRC_out8(2, 0x04);		/* Read plane 2               */
	GRC_out8(GR5_WRITEMODE0, 0x05);
	GRC_out8(GR6_MAP_A0_64, 0x06);
	LEAVE_CRITICAL;
}

static void inline text16_font_leave(text16_font_reg *regs)
{
	IS_CRITICAL;

	ENTER_CRITICAL;
	SEQ_out8(regs->SR2, 0x02);
	SEQ_out8(regs->SR4, 0x04);	/* Restore registers  */
	GRC_out8(regs->GR4, 0x04);
	GRC_out8(regs->GR5, 0x05);
	GRC_out8(regs->GR6, 0x06);
	LEAVE_CRITICAL;
}

/* Need to change this to linear mem, but I haven't seen any problems yet. */
static inline void direct_text16_font_read(int chr, kgiu8 *charcell)
{
	memcpy(charcell, (kgiu8 *)(phys_to_virt(0xA0000)) + (chr << 5), 32);
}

/* And this!! */
static inline void direct_text16_font_write(int chr, kgiu8 *charcell)
{
	memcpy((kgiu8 *)(phys_to_virt(0xA0000)) + (chr << 5), charcell, 32);
}

static void place_pointer(charcell *fb, uint xsize, uint dx, uint dy,
			  uint texty, int isdelt, struct text16_data_s *data)
{
	int i, j;
	uint mask[48];
    kgiu8 font[PCELLTOTAL * 32], *cp;
	text16_font_reg regs;
	charcell *fb1 = fb + POFFSET1, *fb2 = fb + POFFSET2;
	charcell *cell = PDATA;
	uint maxx = PMAXX, maxy = PMAXY;
	int splity = PSPLITY;
    
	if (dy > 16) {
		NOTICE1("PtrHack: Can't deal with characters >16 bits high");
		return;
	}
   
#define LOOPOVERxyi { register int x,y; for (y=0;y<maxy;y++) { \
			i = y * PCELLSIDEX; \
			fb = (y < splity) ? fb1 : fb2; \
			for (x = 0; x < maxx; x++, i++) {
#define ENDLOOPxyi  }}}

    /* Has anything changed since the last time we mucked with the fonts? */ 
	if (!isdelt) {
		LOOPOVERxyi
			isdelt |= (cell[i] != *PCELLOF(fb,xsize,i));
		ENDLOOPxyi
		    
			/* Nope - same characters as we used last time.  Why redo our effort? */
			
			if (!isdelt) {
				LOOPOVERxyi
					CHARSET(PCELLOF(fb, xsize, i), pointer_char[i]);
				ENDLOOPxyi
					return;
			}
	}

    /* Get the characters cells and their associated fonts */
    LOOPOVERxyi
	cell[i] = *PCELLOF(fb, xsize, i);	
    ENDLOOPxyi

    text16_font_enter(&regs);

    LOOPOVERxyi
	direct_text16_font_read(CHAR(cell[i]), font + (i << 5));
    ENDLOOPxyi

    /* Copy the old chars, as a bitmap, into mask[]  */
    memset(mask, 0, sizeof(mask)); 
    LOOPOVERxyi
	cp = font + (i << 5);
	for (j = 0;j < texty; j++) 
		mask[j + y * texty] |= (*(cp++)) << ((3 - x) * 8);
    ENDLOOPxyi

#if DEBUG_LEVEL > 0
    print_mask("Original", mask, texty);
#endif

    /* Mask with the And and Or masks */
    for (i = 0; i < POINTER_MASK_LEN; i++) {
		mask[dy + i] |= (pointer_or_mask[i] >> dx);
		mask[dy + i] ^= (pointer_xor_mask[i] >> dx);
    }
         
#if DEBUG_LEVEL > 0
    print_mask("W/Pointer", mask, texty);
#endif

    /* Now write the new data */
    LOOPOVERxyi
	cp = font + (i << 5);
	for (j = 0; j < texty; j++) 
		*(cp++) = (mask[j + y * texty] >> ((3 - x) * 8)) & 0xFF;
    ENDLOOPxyi

    LOOPOVERxyi
	direct_text16_font_write(pointer_char[i], font + (i << 5));
    ENDLOOPxyi

    text16_font_leave(&regs);

    LOOPOVERxyi
	CHARSET(PCELLOF(fb, xsize, i), pointer_char[i]);
    ENDLOOPxyi
}

static void erase_pointer(charcell *fb, uint xsize, struct text16_data_s *data)
{
	int i;
	charcell *fb1 = fb + POFFSET1, *fb2 = fb + POFFSET2;
	uint maxx = PMAXX, maxy = PMAXY;
	int splity = PSPLITY;

	LOOPOVERxyi
	*PCELLOF(fb, xsize, i) = PDATA[i];
	ENDLOOPxyi
}

#undef LOOPOVERxyi
#undef ENDLOOPxyi

/* Font setting */

static void text16_do_set_font(struct kgi_display *dpy, int no, int from, int to, struct kgi_font *font)
{
	text16_font_reg fontregs;
	int i;
	char *fbuf, *data;
	register int cnt = 32 - font->dy;

	if (font == NULL)
		return;

	/* fbuf changed to point to actual framebuffer and not 0xA0000L */
	fbuf = (void *) dpy->fb.pbuf;
	
	fbuf += (no * 0x2000) + (from << 5);
	data = font->data + font->dy * from;
	if (!data)
		return;

	text16_font_enter(&fontregs);
	/* NOTE: no printk()'s are allowed in here unless klogd is active! */

	for (i = from; i <= to; i++) {
		memmove(fbuf, data, font->dy);
		memset(fbuf + font->dy, 0, cnt);
		fbuf += 32;
		data += font->dy;
	};

	text16_font_leave(&fontregs);

#ifdef CURSOR_HW
	/* Set up cursor */
	CRT_out8((CRT_in8(0x0A) & ~CR0A_CURSOR_START_MASK)
		| ((font->base+1) & CR0A_CURSOR_START_MASK), 0x0A);
	CRT_out8((CRT_in8(0x0B) & ~CR0B_CURSOR_END_MASK)
		| ((font->base+2) & CR0B_CURSOR_END_MASK), 0x0B);
#endif
}

void text16_set_font(struct kgi_display *dpy, struct kgi_font *font,
			    uint from, uint to)
{
	GET_TEXT_DATA;	
	text16_do_set_font(dpy, 0, from, to, font);
	FONTCHANGED = 1;
}

/* This appears to duplicate the general fb_set_origin function in
 * et6000_fb.inc.  This function is currently used for text set_origin. */
void text16_set_origin(struct kgi_display *dpy, uint x, uint y)
{
	GET_TEXT_DATA;
	IS_CRITICAL;
	static uint last_xdot = -1;
	static uint last_ydot = -1;
	register uint xpix = x / dpy->mode.request.dpp.x;
	register uint xdot = x % dpy->mode.request.dpp.x;
	register uint ypix = y / dpy->mode.request.dpp.y;
	register uint ydot = y % dpy->mode.request.dpp.y;

	register uint offset = xpix + ypix*dpy->fb.size.x;

	ENTER_CRITICAL;

	if (ORIGIN != offset) {

		CRT_out8(offset >> 8, CRT_STARTADDR_H);
		CRT_out8(offset, CRT_STARTADDR_L);
		ORIGIN = offset;
	}

	if (last_ydot != ydot) {
		
		CRT_out8(ydot & CR08_ROWSCAN_MASK, CRT_PRESETROW);
		last_ydot = ydot;
	}

	if (last_xdot != xdot) {

		register uint hpan;

		if (dpy->mode.request.dpp.x == 9) {

			hpan = (xdot == 0) ? 0x08 : (xdot-1) & AR13_DATA_MASK;
		} else {

			hpan = xdot & AR13_DATA_MASK;
		}

		WAIT_HVRETRACE();
		ATC_out8(hpan, ATC_HPAN | ATCI_ENABLEDISPLAY);
		last_xdot = xdot;
	}

	LEAVE_CRITICAL;
}

void text16_set_split(struct kgi_display *dpy, int line)
{
	register kgiu8 foo;
	GET_TEXT_DATA;
	
	CRT_out8(line & 0xFF, CRT_LINECOMPARE);

	foo = (line & 0x0100)
		? CRT_in8(0x07) | CR07_LINECOMPARE_B8
		: CRT_in8(0x07) & ~CR07_LINECOMPARE_B8;
	CRT_out8(foo, 0x07);

	foo = (line & 0x0200)
		? CRT_in8(0x09) | CR09_LINECOMPARE_B9
		: CRT_in8(0x09) & ~CR09_LINECOMPARE_B9;
	CRT_out8(foo, 0x09);

	SPLITLINE = line;
}


#define DPPX           (dpy->mode.request.dpp.x)
#define DPPY           (dpy->mode.request.dpp.y)
#define VIRTUALX        (dpy->fb.size.x)
#define VIRTUALY        (dpy->fb.size.y)
#define VISIBLEX        (dpy->mode.request.visible.x)
#define VISIBLEY        (dpy->mode.request.visible.y)

void text16_show_pointer(struct kgi_display *dpy, uint x, uint y)
{
	GET_TEXT_DATA;
	kgiu32 newoff1 = ORIGIN + (x / DPPX) + (y / DPPY) * VIRTUALX;
	kgiu32 newoff2 = 0;
	register charcell *p = (void *) dpy->fb.pbuf;
	uint bitoff = x + y * VISIBLEX;
	uint dx, dy, clipx, clipy, maxx, maxy;
	int splity = PCELLSIDEY - 1;
	
	if (SPLITLINE && SPLITLINE < VIRTUALY * DPPY &&
	    y+POINTER_MASK_LEN > SPLITLINE) {
		newoff2 = (x / DPPX) + ((y - SPLITLINE) / DPPY) * VIRTUALX;
		if (y >= SPLITLINE) {
			newoff1 = newoff2;
			newoff2 = 0;
		}
		else 
			splity = SPLITLINE / DPPY - y / DPPY + 1;
	}

	DEBUG1("PShow(%c):POFFSET1=0x%.8x POFFSET2=0x%.8x PBOFFSET=0x%.8x x=%d,y=%d\n",
		PVISIBLE ? 'V' : 'i', POFFSET1, POFFSET2, PBITOFFSET, x, y);

 	ASSERT((x < VISIBLEX * DPPX) && (y < VISIBLEY * DPPY));

	if (PVISIBLE) 
		erase_pointer(p, VIRTUALX, data);

	dx = x % DPPX;
	dy = y % DPPY;
	clipx = VIRTUALX - (x / DPPX);
	clipy = VIRTUALY - (y / DPPY);

	/* Hmmm; this is hardcoded to 8. */
	maxx = ((POINTER_MASK_WIDTH - 1 + dx) / 8) + 1;
	maxx = (maxx > clipx) ? clipx : maxx;
	maxy = ((POINTER_MASK_LEN - 1 + dy) / DPPY) + 1;
	maxy = (maxy > clipy) ? clipy : maxy;

	if (splity < 0)
		splity = 0;
	else if (splity > maxy)
		splity = maxy;

	POFFSET1 = newoff1;
	POFFSET2 = newoff2;
	PSPLITY = splity;
	PMAXX = maxx;
	PMAXY = maxy;
	
	place_pointer(p, VIRTUALX, dx, dy, DPPY, 
		      bitoff != PBITOFFSET || FONTCHANGED, data);

	PBITOFFSET = bitoff;
	PVISIBLE = 1;
	FONTCHANGED = 0;
}

void text16_hide_pointer(struct kgi_display *dpy)
{
	GET_TEXT_DATA;
	register charcell *p = (void *) dpy->fb.pbuf;

	DEBUG1("PHide(%c): POFFSET1=0x%.8x POFFSET2=0x%.8x PBOFFSET=0x%.8x\n",
		PVISIBLE ? 'V' : 'i', POFFSET1, POFFSET2, PBITOFFSET);

	if (PVISIBLE) {
		erase_pointer(p, VIRTUALX, data);
		PVISIBLE = 0;
	}
}

#define text16_undo_pointer     text16_hide_pointer

#ifdef CURSOR_HW
void text16_show_cursor(struct kgi_display *dpy, uint x, uint y)
{
	GET_TEXT_DATA;
	register kgiu32 newoff;
 	IS_CRITICAL;

	ASSERT((x < VISIBLEX * DPPX) && (y < VISIBLEY * DPPY));

	if (SPLITLINE && y >= SPLITLINE) 
		newoff = (x / DPPX) + ((y - SPLITLINE) / DPPY) * VIRTUALX;
	else
		newoff = ORIGIN + (x / DPPX) + (y / DPPY) * VIRTUALX;

	ENTER_CRITICAL;
	CRT_out8(newoff >> 8, CRT_CURSORADDR_H);
	CRT_out8(newoff & 0xFF, CRT_CURSORADDR_L);
	LEAVE_CRITICAL;
}

void text16_hide_cursor(struct kgi_display *dpy)
{
	IS_CRITICAL;

	ENTER_CRITICAL;
	CRT_out8(0xFF, CRT_CURSORADDR_H);
	CRT_out8(0xFF, CRT_CURSORADDR_L);
	LEAVE_CRITICAL;
}

/* This is badly wrong. Hardware cursor doesn't need an undo function! We 
 * should put NULL here and update kgi.c to actually check if dpy->undo_xxxx
 * are not NULL before calling them.
 */

#else
void text16_show_cursor(struct kgi_display *dpy, uint x, uint y)
{
	GET_TEXT_DATA;
	register kgiu32 newoff;
	register charcell *p = (void *) dpy->fb.pbuf;
	register charcell cell;

	ASSERT((x < VISIBLEX * DPPX) && (y < VISIBLEY * DPPY));

	if (SPLITLINE && y >= SPLITLINE) 
		newoff = (x / DPPX) + ((y - SPLITLINE) / DPPY) * VIRTUALX;
	else
		newoff = ORIGIN + (x / DPPX) + (y / DPPY) * VIRTUALX;

	if (!CVISIBLE || newoff != COFFSET) {
		cell = *(p + newoff);
		ATTRSET(p + newoff, ATTR(cell) ^ CURSOR_MASK);
	} else 
		return;

	if (CVISIBLE)
		*(p + COFFSET) = CDATA;
	else
		CVISIBLE = 1;

	COFFSET = newoff;
	CDATA = cell;
}

void text16_hide_cursor(struct kgi_display *dpy)
{
	GET_TEXT_DATA;
	register kgiu16 *p = (void *) dpy->fb.pbuf;

	if (CVISIBLE)
		*(p + COFFSET) = CDATA;
	
	CVISIBLE = 0;
}

#endif /* CURSOR_HW */

#undef SPLITLINE
#undef ORIGIN

#undef PVISIBLE
#undef POFFSET1
#undef POFFSET2
#undef PSPLITY
#undef PMAXX
#undef PMAXY
#undef PDATA

#undef GET_TEXT_DATA

#ifndef CURSOR_HW
#undef CVISIBLE
#undef COFFSET
#undef CDATA
#endif

